###################################################################
# Chapter 5                                                       #
# Sanchez, J. Time Series for Data Scientists, CUP 2023.          #
# Case Study                                                      #
# Program ch5simcode.R in Base R version                          #
# Last modified: 1/28/2023                                        #
###################################################################


 ##########Chapter 5- Case Study 
######  Working with Italy COVID Data ######
## 
   install.packages("lubridate")
   library(lubridate)

# Read the COVID-19 and Population data 
   covid <- read.csv("covid_19_data.csv")
   head(covid)
   dim(covid)
   populationfile <- read.csv("population_by_country_2020.csv")
   head(populationfile)

## We extract what we need and create a more convenient data file
## The reader can do something similar but for another country when 
## the reader does Exercise 5.21
   
 # First, select only Italy's data 
   covid_italy <- covid[covid$Country.Region=="Italy", ] # Get Italy data
   head(covid_italy,25)
   dim(covid_italy)

   italy_pop <- populationfile[populationfile$Country..or.dependency. == "Italy", "Population..2020."]
   head(italy_pop)
   class(italy_pop)
   
 # Insert the population data into the COVID data file 
   covid_italy$population <- italy_pop

#  Finally we save a file that contains only Italy's information.
   write.csv(covid_italy, file = "covid_italy.csv")

   
#### Restarting with the new smaller file for Italy we created   
   
# We read again that file. 
   casedata=read.csv("covid_italy.csv", header=T) 
   casedata  # small file, we can look at it as a whole 
   str(casedata)
   head(casedata)
   tail(casedata)
   dim(casedata)

    attach(casedata)

### We create now the variable that we are interested in. The number
# of active cases per 100000 population. 

      active= 100000*(Confirmed-Recovered-Deaths)/population
     length(active)

## We create a time index
    inds=seq(as.Date("2020-01-31"), as.Date("2020-05-04"),by= "day")
    class(inds)
    length(inds)
#### by creating an index we can see what day of the 
## year is the first observation. 
     firstday=as.numeric(format(inds[1],"%j"))
      firstday

# Create a ts object 
    covid=ts(active, start=c(2020,firstday),frequency=365)
    class(covid)
    start(covid)
    end(covid)
    
## MAke xts for the plot 
  # install.packages("xts") # uncomment and run if not installed.
   library(xts)
   covid0=xts(active, order.by=inds)
  # we also look at the differenced data 
   covid1=xts(diff(active,1),order.by=inds[2:95])


### Plot the time series to see if it needs to be 
### made stationary 

#    install.packages("dygraphs")
    library("dygraphs")


   ## Figure 5.15 
#png("coronaactive.png", units="in", width=8.5, height=8, res=1200)

   font.axis=2 
    mar=c(5,5,5,5)
   font.main=2
   font.lab=2
   dygraph(covid0,main="Active coronavirus cases", 
        ylab="Active per 100000 population")%>% dyRangeSelector
   #The reader can play with the Range selector at bottom to see windows
   ## of the data and detail by day.
   
   dev.off()  # run only when done viewing the graph 
   
    ## Figure 5.16
#png("coronadiff.png", units="in", width=8.5, height=8, res=1200)

  dygraph(covid1, main="Daily change in active coronavirus cases",
        ylab="Change in active per 100000 population")%>% dyRangeSelector
    dev.off()  # run only when done viewing the graph 
############

   ### Now select the automatic model 
   ## We are using the forecast package now
   #and commands used are from that package. 
   ##After seeing Chapter 6, the reader will see this part more clearly

  library(forecast)
  # Like the authors we get an automatic model 
    model=forecast::auto.arima(covid) # selects the AR, MA or ARMA model needed
    model

# We obtain the fitted values 
     insamplefit=fitted(model)  #using all the data we fit model to data

     # we double check which is the last fitted value
    # notice how auto.arima selects a model for us. Again, Chapter 6 
    # will make that more clear. For now, we just want to use that model 
    # to simulate the future and to also see what the model forecasts. 
     
      end(insamplefit)

## The following code will allow us to set up the simulated future paths 
## derived from the automated model, starting in the last observation
new.starting.value <- c(fitted(model)[c(93:95)]) # or any other value you want 
new.arima.object <- forecast::Arima(new.starting.value, model = model)
# the simulate function will give you the simulated values
## We simulate several paths
sim1=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim11=ts(sim1,start=c(2020,126),freq=365)
sim2=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim22=ts(sim2,start=c(2020,126),freq=365)
sim3=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim33=ts(sim3,start=c(2020,126),freq=365)
sim4=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim44=ts(sim4,start=c(2020,126),freq=365)
sim5=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim55=ts(sim5,start=c(2020,126),freq=365)
sim6=simulate(new.arima.object, nsim = 40, future = T) #this will simulate 40 periods into the future
sim66=ts(sim6,start=c(2020,126),freq=365)
## We could simulate many more paths. 

## We also want to see if the predictions (forecasts) of the 
## model are within the simulated values
predicted=predict(model,40)
predicted$pred


         ########### Figure 5.17 ############

## Now we plot the time series, the 
## fitted values, the simulated future and the forecast
# Notice that because simulations are random possible realizations
# of the fitted model in the future, the reader's curves for 
## the simulations in the plot will look different than Fig. 5.17. 

par(
  mfrow=c(1,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
)

ts.plot(covid,fitted(model), sim11,
        sim22,sim33,sim44,sim55,sim66, predicted$pred, 
        col=c("black", "red","blue",
              "orange", "darkgreen","turquoise2",
              "magenta3","slateblue2", "black"), 
        ylim=c(0,300), lty=c(1,1,3,4,5,6,2,3,1),
        lwd=c(1,1,1,1,1,1,1,1,2),
         ylab="Active covid per 100000",
        main="Several simulated future paths if model is true, \n and shorter term forecast (in bold font)")
date1 <- ymd("2020-05-04")
abline(v=decimal_date(date1))


## Past the vertical line you see the simulations of possible future 
#paths and what the model would have forecasted. 

